@@ -1,11 +1,10 @@
 #!/usr/bin/env python
 
-import os
 import sys
+from pathlib import Path
 
 # Get the project root dir, which is the parent dir of this
-cwd = os.getcwd()
-project_root = os.path.dirname(cwd)
+project_root = str(Path.cwd().parent)
 
 # Insert the project root dir as the first element in the PYTHONPATH.
 # This lets us ensure that the source package is imported, and that its
@@ -36,7 +35,7 @@ master_doc = "index"
 
 # General information about the project.
 project = "Parsel"
-copyright = "2015, Scrapy Project"
+project_copyright = "2015, Scrapy Project"
 
 # The version info for the project you're documenting, acts as replacement
 # for |version| and |release|, also used in various other places throughout
