@@ -1,46 +1,47 @@
-"""
-XPath selectors based on lxml
-"""
+"""XPath and JMESPath selectors based on the lxml and jmespath Python
+packages."""
 
+from __future__ import annotations
+
+import json
 import typing
 import warnings
+from io import BytesIO
 from typing import (
+    TYPE_CHECKING,
     Any,
-    Dict,
-    List,
-    Mapping,
-    Optional,
-    Pattern,
-    Type,
+    Literal,
+    SupportsIndex,
+    TypeAlias,
+    TypedDict,
     TypeVar,
-    Union,
 )
-from warnings import warn
 
-from cssselect import GenericTranslator as OriginalGenericTranslator
+import jmespath
 from lxml import etree, html
-from pkg_resources import parse_version
+from packaging.version import Version
 
 from .csstranslator import GenericTranslator, HTMLTranslator
 from .utils import extract_regex, flatten, iflatten, shorten
 
+if TYPE_CHECKING:
+    from collections.abc import Mapping
+    from re import Pattern
 
-if typing.TYPE_CHECKING:
-    # both require Python 3.8
-    from typing import Literal, SupportsIndex
-
-    # simplified _OutputMethodArg from types-lxml
-    _TostringMethodType = Literal[
-        "html",
-        "xml",
-    ]
+    # typing.Self requires Python 3.11
+    from typing_extensions import Self
 
 
 _SelectorType = TypeVar("_SelectorType", bound="Selector")
-_ParserType = Union[etree.XMLParser, etree.HTMLParser]
+_ParserType: TypeAlias = etree.XMLParser | etree.HTMLParser
+# simplified _OutputMethodArg from types-lxml
+_TostringMethodType = Literal[
+    "html",
+    "xml",
+]
 
-lxml_version = parse_version(etree.__version__)
-lxml_huge_tree_version = parse_version("4.2")
+lxml_version = Version(etree.__version__)
+lxml_huge_tree_version = Version("4.2")
 LXML_SUPPORTS_HUGE_TREE = lxml_version >= lxml_huge_tree_version
 
 
@@ -57,12 +58,18 @@ class CannotDropElementWithoutParent(CannotRemoveElementWithoutParent):
 
 
 class SafeXMLParser(etree.XMLParser):
-    def __init__(self, *args, **kwargs) -> None:
+    def __init__(self, *args: Any, **kwargs: Any) -> None:
         kwargs.setdefault("resolve_entities", False)
         super().__init__(*args, **kwargs)
 
 
-_ctgroup = {
+class CTGroupValue(TypedDict):
+    _parser: type[etree.XMLParser | html.HTMLParser]
+    _csstranslator: GenericTranslator | HTMLTranslator
+    _tostring_method: _TostringMethodType
+
+
+_ctgroup: dict[str, CTGroupValue] = {
     "html": {
         "_parser": html.HTMLParser,
         "_csstranslator": HTMLTranslator(),
@@ -76,80 +83,92 @@ _ctgroup = {
 }
 
 
-def _st(st: Optional[str]) -> str:
-    if st is None:
-        return "html"
-    elif st in _ctgroup:
-        return st
-    else:
-        raise ValueError(f"Invalid type: {st}")
+def _xml_or_html(type_: str | None) -> str:
+    return "xml" if type_ == "xml" else "html"
 
 
 def create_root_node(
     text: str,
-    parser_cls: Type[_ParserType],
-    base_url: Optional[str] = None,
+    parser_cls: type[_ParserType],
+    base_url: str | None = None,
     huge_tree: bool = LXML_SUPPORTS_HUGE_TREE,
+    body: bytes = b"",
+    encoding: str = "utf-8",
 ) -> etree._Element:
     """Create root node for text using given parser class."""
-    body = text.strip().replace("\x00", "").encode("utf8") or b"<html/>"
-    if huge_tree and LXML_SUPPORTS_HUGE_TREE:
-        parser = parser_cls(recover=True, encoding="utf8", huge_tree=True)
-        # the stub wrongly thinks base_url can't be None
-        root = etree.fromstring(body, parser=parser, base_url=base_url)  # type: ignore[arg-type]
+    if not text:
+        body = body.replace(b"\x00", b"").strip()
     else:
-        parser = parser_cls(recover=True, encoding="utf8")
-        root = etree.fromstring(body, parser=parser, base_url=base_url)  # type: ignore[arg-type]
+        body = text.strip().replace("\x00", "").encode(encoding) or b"<html/>"
+
+    if huge_tree and LXML_SUPPORTS_HUGE_TREE:
+        parser = parser_cls(recover=True, encoding=encoding, huge_tree=True)
+        root = etree.fromstring(body, parser=parser, base_url=base_url)
+    else:
+        parser = parser_cls(recover=True, encoding=encoding)
+        root = etree.fromstring(body, parser=parser, base_url=base_url)
         for error in parser.error_log:
             if "use XML_PARSE_HUGE option" in error.message:
                 warnings.warn(
                     f"Input data is too big. Upgrade to lxml "
-                    f"{lxml_huge_tree_version} or later for huge_tree support."
+                    f"{lxml_huge_tree_version} or later for huge_tree support.",
+                    stacklevel=2,
                 )
     if root is None:
         root = etree.fromstring(b"<html/>", parser=parser, base_url=base_url)
     return root
 
 
-class SelectorList(List[_SelectorType]):
+class SelectorList(list[_SelectorType]):
     """
     The :class:`SelectorList` class is a subclass of the builtin ``list``
     class, which provides a few additional methods.
     """
 
     @typing.overload
-    def __getitem__(self, pos: "SupportsIndex") -> _SelectorType:
+    def __getitem__(self, pos: SupportsIndex) -> _SelectorType:
         pass
 
     @typing.overload
-    def __getitem__(self, pos: slice) -> "SelectorList[_SelectorType]":
+    def __getitem__(self, pos: slice) -> SelectorList[_SelectorType]:
         pass
 
     def __getitem__(
-        self, pos: Union["SupportsIndex", slice]
-    ) -> Union[_SelectorType, "SelectorList[_SelectorType]"]:
+        self, pos: SupportsIndex | slice
+    ) -> _SelectorType | SelectorList[_SelectorType]:
         o = super().__getitem__(pos)
         if isinstance(pos, slice):
-            return self.__class__(
-                typing.cast("SelectorList[_SelectorType]", o)
-            )
-        else:
-            return typing.cast(_SelectorType, o)
+            return self.__class__(typing.cast("SelectorList[_SelectorType]", o))
+        return typing.cast("_SelectorType", o)
 
     def __getstate__(self) -> None:
         raise TypeError("can't pickle SelectorList objects")
 
+    def jmespath(self, query: str, **kwargs: Any) -> SelectorList[_SelectorType]:
+        """
+        Call the ``.jmespath()`` method for each element in this list and return
+        their results flattened as another :class:`SelectorList`.
+
+        ``query`` is the same argument as the one in :meth:`Selector.jmespath`.
+
+        Any additional named arguments are passed to the underlying
+        ``jmespath.search`` call, e.g.::
+
+            selector.jmespath('author.name', options=jmespath.Options(dict_cls=collections.OrderedDict))
+        """
+        return self.__class__(flatten([x.jmespath(query, **kwargs) for x in self]))
+
     def xpath(
         self,
         xpath: str,
-        namespaces: Optional[Mapping[str, str]] = None,
-        **kwargs,
-    ) -> "SelectorList[_SelectorType]":
+        namespaces: Mapping[str, str] | None = None,
+        **kwargs: Any,
+    ) -> SelectorList[_SelectorType]:
         """
         Call the ``.xpath()`` method for each element in this list and return
         their results flattened as another :class:`SelectorList`.
 
-        ``query`` is the same argument as the one in :meth:`Selector.xpath`
+        ``xpath`` is the same argument as the one in :meth:`Selector.xpath`
 
         ``namespaces`` is an optional ``prefix: namespace-uri`` mapping (dict)
         for additional prefixes to those registered with ``register_namespace(prefix, uri)``.
@@ -162,12 +181,10 @@ class SelectorList(List[_SelectorType]):
             selector.xpath('//a[href=$url]', url="http://www.example.com")
         """
         return self.__class__(
-            flatten(
-                [x.xpath(xpath, namespaces=namespaces, **kwargs) for x in self]
-            )
+            flatten([x.xpath(xpath, namespaces=namespaces, **kwargs) for x in self])
         )
 
-    def css(self, query: str) -> "SelectorList[_SelectorType]":
+    def css(self, query: str) -> SelectorList[_SelectorType]:
         """
         Call the ``.css()`` method for each element in this list and return
         their results flattened as another :class:`SelectorList`.
@@ -176,9 +193,7 @@ class SelectorList(List[_SelectorType]):
         """
         return self.__class__(flatten([x.css(query) for x in self]))
 
-    def re(
-        self, regex: Union[str, Pattern[str]], replace_entities: bool = True
-    ) -> List[str]:
+    def re(self, regex: str | Pattern[str], replace_entities: bool = True) -> list[str]:
         """
         Call the ``.re()`` method for each element in this list and return
         their results flattened, as a list of strings.
@@ -188,23 +203,21 @@ class SelectorList(List[_SelectorType]):
         Passing ``replace_entities`` as ``False`` switches off these
         replacements.
         """
-        return flatten(
-            [x.re(regex, replace_entities=replace_entities) for x in self]
-        )
+        return flatten([x.re(regex, replace_entities=replace_entities) for x in self])
 
     @typing.overload
     def re_first(
         self,
-        regex: Union[str, Pattern[str]],
+        regex: str | Pattern[str],
         default: None = None,
         replace_entities: bool = True,
-    ) -> Optional[str]:
+    ) -> str | None:
         pass
 
     @typing.overload
     def re_first(
         self,
-        regex: Union[str, Pattern[str]],
+        regex: str | Pattern[str],
         default: str,
         replace_entities: bool = True,
     ) -> str:
@@ -212,10 +225,10 @@ class SelectorList(List[_SelectorType]):
 
     def re_first(
         self,
-        regex: Union[str, Pattern[str]],
-        default: Optional[str] = None,
+        regex: str | Pattern[str],
+        default: str | None = None,
         replace_entities: bool = True,
-    ) -> Optional[str]:
+    ) -> str | None:
         """
         Call the ``.re()`` method for the first element in this list and
         return the result in an string. If the list is empty or the
@@ -230,10 +243,10 @@ class SelectorList(List[_SelectorType]):
         for el in iflatten(
             x.re(regex, replace_entities=replace_entities) for x in self
         ):
-            return el
+            return typing.cast("str", el)
         return default
 
-    def getall(self) -> List[str]:
+    def getall(self) -> list[str]:
         """
         Call the ``.get()`` method for each element is this list and return
         their results flattened, as a list of strings.
@@ -243,14 +256,14 @@ class SelectorList(List[_SelectorType]):
     extract = getall
 
     @typing.overload
-    def get(self, default: None = None) -> Optional[str]:
+    def get(self, default: None = None) -> str | None:
         pass
 
     @typing.overload
     def get(self, default: str) -> str:
         pass
 
-    def get(self, default: Optional[str] = None) -> Optional[str]:
+    def get(self, default: str | None = None) -> Any:
         """
         Return the result of ``.get()`` for the first element in this list.
         If the list is empty, return the default value.
@@ -270,18 +283,6 @@ class SelectorList(List[_SelectorType]):
             return x.attrib
         return {}
 
-    def remove(self) -> None:  # type: ignore[override]
-        """
-        Remove matched nodes from the parent for each element in this list.
-        """
-        warn(
-            "Method parsel.selector.SelectorList.remove is deprecated, please use parsel.selector.SelectorList.drop method instead",
-            category=DeprecationWarning,
-            stacklevel=2,
-        )
-        for x in self:
-            x.remove()
-
     def drop(self) -> None:
         """
         Drop matched nodes from the parent for each element in this list.
@@ -290,15 +291,106 @@ class SelectorList(List[_SelectorType]):
             x.drop()
 
 
+_NOT_SET = object()
+
+
+def _get_root_from_text(text: str, *, type_: str, **lxml_kwargs: Any) -> etree._Element:
+    return create_root_node(text, _ctgroup[type_]["_parser"], **lxml_kwargs)
+
+
+def _get_root_and_type_from_bytes(
+    body: bytes,
+    encoding: str,
+    *,
+    input_type: str | None,
+    **lxml_kwargs: Any,
+) -> tuple[Any, str]:
+    if input_type == "text":
+        return body.decode(encoding), input_type
+    if encoding == "utf-8":
+        try:
+            data = json.load(BytesIO(body))
+        except ValueError:
+            data = _NOT_SET
+        if data is not _NOT_SET:
+            return data, "json"
+    if input_type == "json":
+        return None, "json"
+    assert input_type in ("html", "xml", None)  # nosec
+    type_ = _xml_or_html(input_type)
+    root = create_root_node(
+        text="",
+        body=body,
+        encoding=encoding,
+        parser_cls=_ctgroup[type_]["_parser"],
+        **lxml_kwargs,
+    )
+    return root, type_
+
+
+def _get_root_and_type_from_text(
+    text: str, *, input_type: str | None, **lxml_kwargs: Any
+) -> tuple[Any, str]:
+    if input_type == "text":
+        return text, input_type
+    try:
+        data = json.loads(text)
+    except ValueError:
+        data = _NOT_SET
+    if data is not _NOT_SET:
+        return data, "json"
+    if input_type == "json":
+        return None, "json"
+    assert input_type in ("html", "xml", None)  # nosec
+    type_ = _xml_or_html(input_type)
+    root = _get_root_from_text(text, type_=type_, **lxml_kwargs)
+    return root, type_
+
+
+def _get_root_type(root: Any, *, input_type: str | None) -> str:
+    if isinstance(root, etree._Element):
+        if input_type in {"json", "text"}:
+            raise ValueError(
+                f"Selector got an lxml.etree._Element object as root, "
+                f"and {input_type!r} as type."
+            )
+        return _xml_or_html(input_type)
+    if isinstance(root, (dict, list)) or _is_valid_json(root):
+        return "json"
+    return input_type or "json"
+
+
+def _is_valid_json(text: str) -> bool:
+    try:
+        json.loads(text)
+    except (TypeError, ValueError):
+        return False
+    return True
+
+
+def _load_json_or_none(text: str) -> Any:
+    if isinstance(text, (str, bytes, bytearray)):
+        try:
+            return json.loads(text)
+        except ValueError:
+            return None
+    return None
+
+
 class Selector:
-    """
-    :class:`Selector` allows you to select parts of an XML or HTML text using CSS
-    or XPath expressions and extract data from it.
+    """Wrapper for input data in HTML, JSON, or XML format, that allows
+    selecting parts of it using selection expressions.
 
-    ``text`` is a `str`` object
+    You can write selection expressions in CSS or XPath for HTML and XML
+    inputs, or in JMESPath for JSON inputs.
 
-    ``type`` defines the selector type, it can be ``"html"``, ``"xml"`` or ``None`` (default).
-    If ``type`` is ``None``, the selector defaults to ``"html"``.
+    ``text`` is an ``str`` object.
+
+    ``body`` is a ``bytes`` object. It can be used together with the
+    ``encoding`` argument instead of the ``text`` argument.
+
+    ``type`` defines the selector type. It can be ``"html"`` (default),
+    ``"json"``, or ``"xml"``.
 
     ``base_url`` allows setting a URL for the document. This is needed when looking up external entities with relative paths.
     See the documentation for :func:`lxml.etree.fromstring` for more information.
@@ -313,18 +405,16 @@ class Selector:
     """
 
     __slots__ = [
-        "text",
-        "namespaces",
-        "type",
-        "_expr",
-        "root",
         "__weakref__",
-        "_parser",
-        "_csstranslator",
-        "_tostring_method",
+        "_expr",
+        "_huge_tree",
+        "_text",
+        "body",
+        "namespaces",
+        "root",
+        "type",
     ]
 
-    _default_type: Optional[str] = None
     _default_namespaces = {
         "re": "http://exslt.org/regular-expressions",
         # supported in libxslt:
@@ -340,58 +430,141 @@ class Selector:
 
     def __init__(
         self,
-        text: Optional[str] = None,
-        type: Optional[str] = None,
-        namespaces: Optional[Mapping[str, str]] = None,
-        root: Optional[Any] = None,
-        base_url: Optional[str] = None,
-        _expr: Optional[str] = None,
+        text: str | None = None,
+        type: str | None = None,  # noqa: A002
+        body: bytes | bytearray = b"",
+        encoding: str = "utf-8",
+        namespaces: Mapping[str, str] | None = None,
+        root: Any | None = _NOT_SET,
+        base_url: str | None = None,
+        _expr: str | None = None,
         huge_tree: bool = LXML_SUPPORTS_HUGE_TREE,
     ) -> None:
-        self.type = st = _st(type or self._default_type)
-        self._parser: Type[_ParserType] = typing.cast(
-            Type[_ParserType], _ctgroup[st]["_parser"]
-        )
-        self._csstranslator: OriginalGenericTranslator = typing.cast(
-            OriginalGenericTranslator, _ctgroup[st]["_csstranslator"]
-        )
-        self._tostring_method: "_TostringMethodType" = typing.cast(
-            "_TostringMethodType", _ctgroup[st]["_tostring_method"]
-        )
+        self.root: Any
+        if type not in ("html", "json", "text", "xml", None):
+            raise ValueError(f"Invalid type: {type}")
+
+        if text is None and not body and root is _NOT_SET:
+            raise ValueError("Selector needs text, body, or root arguments")
+
+        if text is not None and not isinstance(text, str):
+            msg = f"text argument should be of type str, got {text.__class__}"
+            raise TypeError(msg)
 
         if text is not None:
+            if root is not _NOT_SET:
+                warnings.warn(
+                    "Selector got both text and root, root is being ignored.",
+                    stacklevel=2,
+                )
             if not isinstance(text, str):
                 msg = f"text argument should be of type str, got {text.__class__}"
                 raise TypeError(msg)
-            root = self._get_root(text, base_url, huge_tree)
-        elif root is None:
-            raise ValueError("Selector needs either text or root argument")
+
+            root, type = _get_root_and_type_from_text(  # noqa: A001
+                text,
+                input_type=type,
+                base_url=base_url,
+                huge_tree=huge_tree,
+            )
+            self.root = root
+            self.type = type
+        elif body:
+            if not isinstance(body, (bytes, bytearray)):
+                msg = f"body argument should be of type bytes or bytearray, got {body.__class__}"
+                raise TypeError(msg)
+            root, type = _get_root_and_type_from_bytes(  # noqa: A001
+                body=bytes(body),
+                encoding=encoding,
+                input_type=type,
+                base_url=base_url,
+                huge_tree=huge_tree,
+            )
+            self.root = root
+            self.type = type
+        elif root is _NOT_SET:
+            raise ValueError("Selector needs text, body, or root arguments")
+        else:
+            self.root = root
+            self.type = _get_root_type(root, input_type=type)
 
         self.namespaces = dict(self._default_namespaces)
         if namespaces is not None:
             self.namespaces.update(namespaces)
-        self.root = root
+
         self._expr = _expr
+        self._huge_tree = huge_tree
+        self._text = text
 
     def __getstate__(self) -> Any:
         raise TypeError("can't pickle Selector objects")
 
     def _get_root(
         self,
-        text: str,
-        base_url: Optional[str] = None,
+        text: str = "",
+        base_url: str | None = None,
         huge_tree: bool = LXML_SUPPORTS_HUGE_TREE,
+        type_: str | None = None,
+        body: bytes = b"",
+        encoding: str = "utf-8",
     ) -> etree._Element:
         return create_root_node(
-            text, self._parser, base_url=base_url, huge_tree=huge_tree
+            text,
+            body=body,
+            encoding=encoding,
+            parser_cls=_ctgroup[type_ or self.type]["_parser"],
+            base_url=base_url,
+            huge_tree=huge_tree,
         )
 
-    def xpath(
-        self: _SelectorType,
+    def jmespath(
+        self,
         query: str,
-        namespaces: Optional[Mapping[str, str]] = None,
-        **kwargs,
-    ) -> SelectorList[_SelectorType]:
+        **kwargs: Any,
+    ) -> SelectorList[Self]:
+        """
+        Find objects matching the JMESPath ``query`` and return the result as a
+        :class:`SelectorList` instance with all elements flattened. List
+        elements implement :class:`Selector` interface too.
+
+        ``query`` is a string containing the `JMESPath
+        <https://jmespath.org/>`_ query to apply.
+
+        Any additional named arguments are passed to the underlying
+        ``jmespath.search`` call, e.g.::
+
+            selector.jmespath('author.name', options=jmespath.Options(dict_cls=collections.OrderedDict))
+        """
+        if self.type == "json":
+            if isinstance(self.root, str):
+                # Selector received a JSON string as root.
+                data = _load_json_or_none(self.root)
+            else:
+                data = self.root
+        else:
+            assert self.type in {"html", "xml"}  # nosec
+            data = _load_json_or_none(self.root.text)
+
+        result = jmespath.search(query, data, **kwargs)
+        if result is None:
+            result = []
+        elif not isinstance(result, list):
+            result = [result]
+
+        def make_selector(x: Any) -> Selector:  # closure function
+            if isinstance(x, str):
+                return self.__class__(text=x, _expr=query, type="text")
+            return self.__class__(root=x, _expr=query)
+
+        result = [make_selector(x) for x in result]
+        return typing.cast("SelectorList[Self]", self.selectorlist_cls(result))
+
+    def xpath(
+        self,
+        query: str,
+        namespaces: Mapping[str, str] | None = None,
+        **kwargs: Any,
+    ) -> SelectorList[Self]:
         """
         Find nodes matching the xpath ``query`` and return the result as a
         :class:`SelectorList` instance with all elements flattened. List
@@ -409,12 +582,18 @@ class Selector:
 
             selector.xpath('//a[href=$url]', url="http://www.example.com")
         """
-        try:
-            xpathev = self.root.xpath
-        except AttributeError:
-            return typing.cast(
-                SelectorList[_SelectorType], self.selectorlist_cls([])
-            )
+        if self.type not in ("html", "xml", "text"):
+            raise ValueError(f"Cannot use xpath on a Selector of type {self.type!r}")
+        if self.type in ("html", "xml"):
+            try:
+                xpathev = self.root.xpath
+            except AttributeError:
+                return typing.cast("SelectorList[Self]", self.selectorlist_cls([]))
+        else:
+            try:
+                xpathev = self._get_root(self._text or "", type_="html").xpath
+            except AttributeError:
+                return typing.cast("SelectorList[Self]", self.selectorlist_cls([]))
 
         nsp = dict(self.namespaces)
         if namespaces is not None:
@@ -429,20 +608,21 @@ class Selector:
         except etree.XPathError as exc:
             raise ValueError(f"XPath error: {exc} in {query}")
 
-        if type(result) is not list:
+        if not isinstance(result, list):
             result = [result]
 
         result = [
             self.__class__(
-                root=x, _expr=query, namespaces=self.namespaces, type=self.type
+                root=x,
+                _expr=query,
+                namespaces=self.namespaces,
+                type=_xml_or_html(self.type),
             )
             for x in result
         ]
-        return typing.cast(
-            SelectorList[_SelectorType], self.selectorlist_cls(result)
-        )
+        return typing.cast("SelectorList[Self]", self.selectorlist_cls(result))
 
-    def css(self: _SelectorType, query: str) -> SelectorList[_SelectorType]:
+    def css(self, query: str) -> SelectorList[Self]:
         """
         Apply the given CSS selector and return a :class:`SelectorList` instance.
 
@@ -453,14 +633,15 @@ class Selector:
 
         .. _cssselect: https://pypi.python.org/pypi/cssselect/
         """
+        if self.type not in ("html", "xml", "text"):
+            raise ValueError(f"Cannot use css on a Selector of type {self.type!r}")
         return self.xpath(self._css2xpath(query))
 
     def _css2xpath(self, query: str) -> str:
-        return self._csstranslator.css_to_xpath(query)
+        type_ = _xml_or_html(self.type)
+        return _ctgroup[type_]["_csstranslator"].css_to_xpath(query)
 
-    def re(
-        self, regex: Union[str, Pattern[str]], replace_entities: bool = True
-    ) -> List[str]:
+    def re(self, regex: str | Pattern[str], replace_entities: bool = True) -> list[str]:
         """
         Apply the given regex and return a list of strings with the
         matches.
@@ -473,23 +654,22 @@ class Selector:
         Passing ``replace_entities`` as ``False`` switches off these
         replacements.
         """
-        return extract_regex(
-            regex, self.get(), replace_entities=replace_entities
-        )
+        data = self.get()
+        return extract_regex(regex, data, replace_entities=replace_entities)
 
     @typing.overload
     def re_first(
         self,
-        regex: Union[str, Pattern[str]],
+        regex: str | Pattern[str],
         default: None = None,
         replace_entities: bool = True,
-    ) -> Optional[str]:
+    ) -> str | None:
         pass
 
     @typing.overload
     def re_first(
         self,
-        regex: Union[str, Pattern[str]],
+        regex: str | Pattern[str],
         default: str,
         replace_entities: bool = True,
     ) -> str:
@@ -497,10 +677,10 @@ class Selector:
 
     def re_first(
         self,
-        regex: Union[str, Pattern[str]],
-        default: Optional[str] = None,
+        regex: str | Pattern[str],
+        default: str | None = None,
         replace_entities: bool = True,
-    ) -> Optional[str]:
+    ) -> str | None:
         """
         Apply the given regex and return the first string which matches. If
         there is no match, return the default value (``None`` if the argument
@@ -516,29 +696,32 @@ class Selector:
             default,
         )
 
-    def get(self) -> str:
+    def get(self) -> Any:
         """
-        Serialize and return the matched nodes in a single string.
-        Percent encoded content is unquoted.
+        Serialize and return the matched nodes.
+
+        For HTML and XML, the result is always a string, and percent-encoded
+        content is unquoted.
         """
+        if self.type in ("text", "json"):
+            return self.root
         try:
             return etree.tostring(
                 self.root,
-                method=self._tostring_method,
+                method=_ctgroup[self.type]["_tostring_method"],
                 encoding="unicode",
                 with_tail=False,
             )
         except (AttributeError, TypeError):
             if self.root is True:
                 return "1"
-            elif self.root is False:
+            if self.root is False:
                 return "0"
-            else:
-                return str(self.root)
+            return str(self.root)
 
     extract = get
 
-    def getall(self) -> List[str]:
+    def getall(self) -> list[str]:
         """
         Serialize and return the matched node in a 1-element list of strings.
         """
@@ -556,50 +739,22 @@ class Selector:
         """
         Remove all namespaces, allowing to traverse the document using
         namespace-less xpaths. See :ref:`removing-namespaces`.
+        For JSON selectors, this method does nothing.
         """
+        if self.type == "json":
+            return
+
         for el in self.root.iter("*"):
             if el.tag.startswith("{"):
                 el.tag = el.tag.split("}", 1)[1]
             # loop on element attributes also
             for an in el.attrib:
                 if an.startswith("{"):
-                    # this cast shouldn't be needed as pop never returns None
-                    el.attrib[an.split("}", 1)[1]] = typing.cast(
-                        str, el.attrib.pop(an)
-                    )
+                    el.attrib[an.split("}", 1)[1]] = el.attrib.pop(an)
         # remove namespace declarations
         etree.cleanup_namespaces(self.root)
 
-    def remove(self) -> None:
-        """
-        Remove matched nodes from the parent element.
-        """
-        warn(
-            "Method parsel.selector.Selector.remove is deprecated, please use parsel.selector.Selector.drop method instead",
-            category=DeprecationWarning,
-            stacklevel=2,
-        )
-        try:
-            parent = self.root.getparent()
-        except AttributeError:
-            # 'str' object has no attribute 'getparent'
-            raise CannotRemoveElementWithoutRoot(
-                "The node you're trying to remove has no root, "
-                "are you trying to remove a pseudo-element? "
-                "Try to use 'li' as a selector instead of 'li::text' or "
-                "'//li' instead of '//li/text()', for example."
-            )
-
-        try:
-            parent.remove(self.root)  # type: ignore[union-attr]
-        except AttributeError:
-            # 'NoneType' object has no attribute 'remove'
-            raise CannotRemoveElementWithoutParent(
-                "The node you're trying to remove has no parent, "
-                "are you trying to remove a root element?"
-            )
-
-    def drop(self):
+    def drop(self) -> None:
         """
         Drop matched nodes from the parent element.
         """
@@ -616,9 +771,11 @@ class Selector:
 
         try:
             if self.type == "xml":
+                if parent is None:
+                    raise ValueError("This node has no parent")
                 parent.remove(self.root)
             else:
-                self.root.drop_tree()
+                typing.cast("html.HtmlElement", self.root).drop_tree()
         except (AttributeError, AssertionError):
             # 'NoneType' object has no attribute 'drop'
             raise CannotDropElementWithoutParent(
@@ -627,8 +784,14 @@ class Selector:
             )
 
     @property
-    def attrib(self) -> Dict[str, str]:
-        """Return the attributes dictionary for underlying element."""
+    def attrib(self) -> dict[str, str]:
+        """
+        Return the attributes dictionary for underlying element.
+        For JSON selectors, return an empty dict.
+        """
+        if self.type == "json":
+            return {}
+
         return dict(self.root.attrib)
 
     def __bool__(self) -> bool:
@@ -642,7 +805,8 @@ class Selector:
     __nonzero__ = __bool__
 
     def __str__(self) -> str:
-        data = repr(shorten(self.get(), width=40))
-        return f"<{type(self).__name__} xpath={self._expr!r} data={data}>"
+        return str(self.get())
 
-    __repr__ = __str__
+    def __repr__(self) -> str:
+        data = repr(shorten(str(self.get()), width=40))
+        return f"<{type(self).__name__} query={self._expr!r} data={data}>"
