@@ -1,9 +1,15 @@
+from __future__ import annotations
+
 import re
-from typing import Any, List, Pattern, Union, cast, Match
+from typing import TYPE_CHECKING, Any, cast
+
 from w3lib.html import replace_entities as w3lib_replace_entities
 
+if TYPE_CHECKING:
+    from collections.abc import Iterable, Iterator
 
-def flatten(x):
+
+def flatten(x: Iterable[Any]) -> list[Any]:
     """flatten(sequence) -> list
     Returns a single, flat list which contains all elements retrieved
     from the sequence and all recursively contained sub-sequences
@@ -21,9 +27,13 @@ def flatten(x):
     return list(iflatten(x))
 
 
-def iflatten(x):
+def iflatten(x: Iterable[Any]) -> Iterator[Any]:
     """iflatten(sequence) -> Iterator
-    Similar to ``.flatten()``, but returns iterator instead"""
+    Similar to ``.flatten()``, but returns iterator instead
+    Examples:
+    >>> list(iflatten([[1, 2], (3, 4)]))
+    [1, 2, 3, 4]
+    """
     for el in x:
         if _is_listlike(el):
             yield from flatten(el)
@@ -56,8 +66,8 @@ def _is_listlike(x: Any) -> bool:
 
 
 def extract_regex(
-    regex: Union[str, Pattern[str]], text: str, replace_entities: bool = True
-) -> List[str]:
+    regex: str | re.Pattern[str], text: str, replace_entities: bool = True
+) -> list[str]:
     """Extract a list of strings from the given text/encoding using the following policies:
     * if the regex contains a named group called "extract" that will be returned
     * if the regex contains multiple numbered groups, all those will be returned (flattened)
@@ -69,7 +79,7 @@ def extract_regex(
     if "extract" in regex.groupindex:
         # named group
         try:
-            extracted = cast(Match[str], regex.search(text)).group("extract")
+            extracted = cast("re.Match[str]", regex.search(text)).group("extract")
         except AttributeError:
             strings = []
         else:
